/**********************************************************************
 * Copyright (c) 2013 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: SecureConnectionImpl.java,v 1.12 2009-09-28 19:17:47 paules Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.hyades.internal.execution.local.control;

import java.security.Provider;
import java.security.Security;
import java.security.Provider.Service;

import javax.net.ssl.SSLContext;

import org.eclipse.hyades.internal.execution.local.common.HyadesExecMessageDebug;

/**  
 * Note: Similar code exists in the following three classes:
 * 			org.eclipse.tptp.platform.agentcontroller.jsse.SSLFactory
 * 			org.eclipse.tptp.platform.execution.client.core.internal.SecureConnectionImpl
 * 			org.eclipse.hyades.internal.execution.local.control.SecurityProtocol
 * Any changes made to the following methods need to be replicated in all three classes.
 */
public class SecurityProtocol {
	private static SecurityProtocol _instance;
	private String _defaultProtocol="SSL";
	private	static String _userProvidedProtocol;
	private static String _protocol=null;
	
	private SecurityProtocol() {
	}
	
	public static SecurityProtocol getInstance() {
		if(_instance == null){
			_instance = new SecurityProtocol();
		}
		return _instance;
	}
	
	
	public String getClientProtocol() {
		if (_protocol != null) return _protocol;
		
		_userProvidedProtocol=System.getProperty("SECURITY_SOCKET_PROTOCOL");
		
		/* Log data if debug tracing is enabled */
		if(HyadesExecMessageDebug.HYADES_EXEC_MSG_DEBUG && HyadesExecMessageDebug.HYADES_DATA_DEBUG) {
			HyadesExecMessageDebug.writeDebugMessage("userProvidedProtocol: " + _userProvidedProtocol);
		}
		
		if (_userProvidedProtocol!=null) {
			// use the protocol supplied by the user if it is valid
			_protocol = validate(_userProvidedProtocol) ? _userProvidedProtocol : _defaultProtocol;
		}
		else {
			_protocol = (isIBMJVM() && validate("SSL_TLSv2")) ? "SSL_TLSv2" : _defaultProtocol;
		}
		return _protocol;
	}
	
	public String getServerProtocol() {
		if (_protocol != null) return _protocol;
		
		_userProvidedProtocol=System.getenv("SECURITY_SOCKET_PROTOCOL");
		
		if (_userProvidedProtocol!=null) {
			// use the protocol supplied by the user if it is valid
			_protocol = validate(_userProvidedProtocol) ? _userProvidedProtocol : _defaultProtocol;
		}
		else {
			_protocol = (isIBMJVM() && validate("SSL_TLSv2")) ? "SSL_TLSv2" : _defaultProtocol;
		}
		return _protocol;
	}
	
	/**
	 * Checks if the protocol is supported by the JVM used by the agent controller 
	 */
	public boolean validate(String protocol) {
		//ArrayList<String> protocolsList = getJVMSupportedProtocols();
		
		// Check for any of the other valid algorithms
		if(protocol.equalsIgnoreCase("default")) {
			return false;
		}		
		
		Provider[] providers = Security.getProviders();
		for (int p = 0; p < providers.length; p++) {
			Provider provider = providers[p];
			//System.out.println("Provider[" + p + "] = " + providers[p]);
			// see if this provider supports the provided security protocol
			Service service = provider.getService(SSLContext.class.getSimpleName(), protocol);
			if(service != null) {
				return true;
			}
		}
		return false;
	}
	
	public boolean isIBMJVM(){
		if (System.getProperty("java.vm.vendor").indexOf("IBM") >= 0) {
			//String version = System.getProperty("java.version");
			return true;
		}
		return false;
	}
	
	public boolean isOracleJVM() {
		if (System.getProperty("java.vm.vendor").toLowerCase().indexOf("oracle") >= 0)
			return true;
			
		return false;
	}
	public String[] removeHelloMsgs(String[] protos) {
		int counter=0;
		for (String item : protos) {
			if (item.toLowerCase().indexOf("hello") == -1) {
				counter++;							
			}
		}
		String []selProtos = new String [counter];
		counter=0;
		for (String item : protos) {
			if (item.toLowerCase().indexOf("hello") == -1) {
				selProtos[counter++] = item;							
			}
		}
		
		return selProtos;
	}
	
	/*	Code to get protocols supported by the JVM
	public ArrayList<String> getJVMSupportedProtocols() {
		ArrayList<String> supportedProtocols = new ArrayList<String>();
		SSLContext sslCtx = null;
		
		try {
			sslCtx = SSLContext.getInstance("DEFAULT");
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
		}
		Provider provider = sslCtx.getProvider();
		Set<Service> services = provider.getServices();
		Iterator<Service> sIt = services.iterator();
		while(sIt.hasNext()) {
			Service srv = sIt.next();
			if (provider.getService(SSLContext.class.getSimpleName(), srv.getAlgorithm()) != null && !srv.getAlgorithm().equalsIgnoreCase("default")) {
				supportedProtocols.add(srv.getAlgorithm());
			}
		} 
		
		return supportedProtocols;
	}
	*/

}

