/**********************************************************************
 * Copyright (c) 2003, 2012 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: XMLmonWaitLoader.java,v 1.11 2010-05-11 16:30:04 jwest Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 * Intel - loader is extended for contention analysis support  
 
 * $Id: XMLmonWaitLoader.java,v 1.11 2010-05-11 16:30:04 jwest Exp $
 **********************************************************************/
package org.eclipse.hyades.loaders.trace;
import org.eclipse.hyades.loaders.hierarchy.Constants;
import org.eclipse.hyades.loaders.util.HierarchyContext;
import org.eclipse.hyades.loaders.util.LoadersUtils;
import org.eclipse.hyades.loaders.util.LookupServiceExtensions;
import org.eclipse.hyades.models.trace.TRCHeapObject;
import org.eclipse.hyades.models.trace.TRCThreadWaitingForObjectEvent;
import org.eclipse.hyades.models.trace.TraceFactory;
import org.eclipse.hyades.models.trace.TracePackage;
import org.eclipse.hyades.models.trace.impl.TRCFullHeapObjectImpl;
import org.eclipse.hyades.models.trace.impl.TRCHeapObjectImpl;
public class XMLmonWaitLoader extends TraceXMLThreadEventsLoader {
	//~ Static fields/initializers
	// -----------------------------------------------------------------
	protected static final String TIMEOUT = "timeout";
	//~ Instance fields
	// ----------------------------------------------------------------------------
	protected long timeout;
	//~ Methods
	// ------------------------------------------------------------------------------------
	public void addAttribute(String name, String value) {
		switch (LoadersUtils.getHashCode(name)) {
			case TraceConstants.TIMEOUT_int :
				timeout = Long.parseLong(value);
				break;
			default :
				super.addAttribute(name, value);
				break;
		}
	}
	public void addYourselfInContext() {
		if(context.isFilterOn())
		{
			if(context.getFilterEngine().isFiltered(TracePackage.eINSTANCE.getTRCThread_Name(),LoadersUtils.getLookUpKey(threadIdRef)))
			{
				return;
			}
		}
		
		theProcess = getProcess();
		theThread = getThreadByIdRef(theProcess);
		dispatchProcessMode(ProcessSteps.ALL);
		
		if (objIdRef == -1) {
			// This was the previous way of detecting sleep, which was found to either not have worked, or to no longer work, when I tested on various platforms/JVMs
//			TRCThreadSleepingEvent sleepingEvent = TraceFactory.eINSTANCE.createTRCThreadSleepingEvent();
//			sleepingEvent.setSleepingTime(timeout);
//			sleepingEvent.setTime(createDeltaTime());//time);
//			sleepingEvent.setThread(theThread);
//			addAnnotationsIfRequired(sleepingEvent);
//			theThread.getThreadEvents().add(sleepingEvent);
		} else {
			dispatchProcessMode(ProcessSteps.ALL);
			TRCThreadWaitingForObjectEvent threadWaitingForObjectEvent = TraceFactory.eINSTANCE.createTRCThreadWaitingForObjectEvent();
			threadWaitingForObjectEvent.setObjectWaitingFor(theObject);
			threadWaitingForObjectEvent.setThread(theThread);
			threadWaitingForObjectEvent.setTime(createDeltaTime());//time);
			threadWaitingForObjectEvent.setTimeout(timeout);
			addAnnotationsIfRequired(threadWaitingForObjectEvent);
			theThread.getThreadEvents().add(threadWaitingForObjectEvent);
		}
	}
	public void initialize(HierarchyContext context, String name) {
		loadToModel = context.isLoadToModel();
		super.initialize(context, name);
		timeout = 0;
		annotations.clear();
		currentAnnotationValueEntryActive=false;
	}

	@SuppressWarnings("rawtypes")
	protected void processHF(int step) {
		super.processHF(step);
		Class clazz = (TraceUtils.isBooleanOptionEnabled(context,Constants.MULTIPLE_HEAP_DUMPS) ? TRCFullHeapObjectImpl.class : TRCHeapObjectImpl.class);
		if ((theObject == null) || !(theObject instanceof TRCHeapObject)) {
			theObject = (TRCHeapObject) LookupServiceExtensions.getInstance().locate(context, clazz, objIdRef);
		}
	}
}