/*******************************************************************************
 * Copyright (c) 2005, 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials 
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: AddConfigPropertyAction.java,v 1.6 2007-03-22 14:42:06 paules Exp $
 * 
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.hyades.test.ui.internal.action;

import org.eclipse.hyades.models.common.configuration.CFGComparableProperty;
import org.eclipse.hyades.models.common.configuration.CFGPropertyGroup;
import org.eclipse.hyades.models.common.configuration.Common_ConfigurationPackage;
import org.eclipse.hyades.test.ui.internal.dialog.ComparablePropertyInputDialog;
import org.eclipse.hyades.test.ui.internal.model.ui.AddChildAction;
import org.eclipse.hyades.test.ui.internal.resources.UiPluginResourceBundle;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;

/**
 * This class handles the add action in {@link CFGConfigurableObject} editing page to add new 
 * {@link CFGComparableProperty}. Subclass can override {@link #addProperties()} to do specific
 * actions when the add button is selected.
 * 
 * @author bjiang
 * @since  1.3
 */
public class AddConfigPropertyAction extends AddChildAction
{
	private CFGPropertyGroup parent;
	
	/**
	 * Default Constructor for AddConfigPropertyAction
	 */
	public AddConfigPropertyAction()
	{
		super(Common_ConfigurationPackage.eINSTANCE.getCFGPropertyGroup(), UiPluginResourceBundle.BTN_ADD);	 
	}
	
	/**
	 * Constructor for AddConfigPropertyAction
	 * @param parent the CFGPropertyGroup the new property belongs to
	 * @param image the image for the property this action adds, or <code>null</code> if no image.
	 */
	public AddConfigPropertyAction(CFGPropertyGroup parent, ImageDescriptor image)
	{
		super(Common_ConfigurationPackage.eINSTANCE.getCFGPropertyGroup(), UiPluginResourceBundle.BTN_ADD); 
		this.parent = parent;
		super.setParent(parent);
		if( image != null )
			super.setImageDescriptor(image);	
	}

	/**
	 * Sets the default CFGPropertyGroup that is returned by 
	 * {@link #getParentPropertyGroup(IStructuredSelection)}.  If <code>null</code> then the
	 * selection is analysed in order to identify the CFGPropertyGroup.
	 * @param parent
	 */
	public void setPropertyGroup(CFGPropertyGroup parent)
	{
		this.parent = parent;
		super.setParent(parent);
	}

	/**
	 * Returns the CFGPropertyGroup that contains the first element of the structured 
	 * selection if there is no default property group or <code>null</code>.
	 * @param structuredSelection
	 * @return CFGPropertyGroup
	 */
	protected CFGPropertyGroup getPropertyGroup(IStructuredSelection structuredSelection)
	{
		if( parent == null )
		    parent = (CFGPropertyGroup)super.getParent(structuredSelection);
		
		return parent;
	}	
	
	/**
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run()
	{
		setActionPerformed(false);
		CFGComparableProperty[] newProperties = addProperties();
		if(newProperties == null || newProperties.length == 0)
			return;
			
		if(getPropertyGroup(getStructuredSelection()) == null)
		    return;
		    
		for(int i = 0; i < newProperties.length; i++)
		{				
		    if(newProperties[i] != null)
		        parent.getProperties().add(newProperties[i]);
		}
	}
	
	/**
	 * Adds one or more {@link org.eclipse.hyades.models.common.configuration.CFGComparableProperty}.
	 * 
	 * @return new properties created or empty array if none.
	 */
	protected CFGComparableProperty[] addProperties()
	{
		CFGComparableProperty[] newProperties = null;
		ComparablePropertyInputDialog dialog = new ComparablePropertyInputDialog(Display.getCurrent().getActiveShell(), UiPluginResourceBundle.DLG_NEW_PROPERTY); 
		dialog.open();
		
		CFGComparableProperty newProperty = dialog.getProperty();	
		if(newProperty != null)
		{
			newProperties = new CFGComparableProperty[1];
			newProperties[0] = newProperty;
		}
		else
		    newProperties = new CFGComparableProperty[0];
		
		return newProperties;
	}

}
