/**********************************************************************
 * Copyright (c) 2005, 2012 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: TraceCallStack.java,v 1.1 2005-07-22 14:49:35 hleung Exp $
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/
package org.eclipse.hyades.uml2sd.trace.loaders.internal;

import java.util.Stack;

import org.eclipse.hyades.models.trace.TRCFullMethodInvocation;
import org.eclipse.hyades.models.trace.TRCThread;
import org.eclipse.hyades.uml2sd.trace.loaders.BaseTraceInteractions;
import org.eclipse.hyades.uml2sd.trace.util.TIUtils;


/**
 * Thread call stack information is kept up to date here during Trace Interaction build.
 */
public class TraceCallStack {
	
	private final BaseTraceInteractions interactions;
	
	private TRCThread eThread;
	private Stack<TraceMessageAndCallAssociation> callStack;
	
	/**
	 * @param interactions
	 * @param eThread
	 */
	public TraceCallStack(BaseTraceInteractions interactions_, TRCThread eThread) {
		setEThread(eThread);
		interactions = interactions_;
		callStack = new Stack<TraceMessageAndCallAssociation>();
	}
	
	/**
	 * @param eThread
	 */
	public void setEThread(TRCThread eThread) {
		this.eThread = eThread;
	}
	
	/**
	 * @return
	 */
	public TRCThread getEThread() {
		return eThread;
	}
	
	/**
	 * @param methodInvocation
	 */
	public void pushCall(TraceSyncMessage syncMessage, TRCFullMethodInvocation methodInvocation) {
		callStack.push(new TraceMessageAndCallAssociation(syncMessage, methodInvocation));
		interactions.sortTraceThreadList(this);
	}
	
	/**
	 * @return
	 */
	public TraceMessageAndCallAssociation popCall() {
		if (callStack.size() == 0) {
			return null;
		}
		TraceMessageAndCallAssociation ret = callStack.pop();
		this.interactions.sortTraceThreadList(this);
		return ret;
	}
	
	/**
	 * @return
	 */
	public TraceMessageAndCallAssociation getCurrentCall() {
		if (callStack.size() > 0) {
			return callStack.lastElement();
		}
		return null;
	}
	
	/**
	 * @return
	 */
	public double getCurrentExitTime() {
		if (callStack.size() > 0) {
			double ret = TIUtils.getAbsoluteExitTime(getCurrentCall().getMethodInvocation());
			if (ret > 0.0) {
				return ret;
			} else {
				return Double.MAX_VALUE / 10;
			}
		}
		return Double.MAX_VALUE;
	}

	/**
	 * @return
	 */
	public long getCurrentTicket() {
		if (callStack.size() > 0) {
			return getCurrentCall().getMethodInvocation().getTicket();
		}
		return Long.MAX_VALUE;
	}

	/**
	 * @return
	 */
	public short getCurrentStackDepth() {
		if (callStack.size() > 0) {
			return getCurrentCall().getMethodInvocation().getStackDepth();
		}
		return Short.MAX_VALUE;
	}
}