/**********************************************************************
 * Copyright (c) 2006, 2013 IBM Corporation and others.
 * All rights reserved.   This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * $Id: ChkPass.c,v 1.9 2010-01-28 20:26:52 jwest Exp $
 *
 * Contributors:
 * IBM - Initial API and implementation
 **********************************************************************/

#include <stdio.h>
#include <stdlib.h>

#if defined(__linux__) || defined(__sparc) || defined(_SOLARISX86)
	#include <pwd.h>
	#include <shadow.h>
	#include <string.h>
	#include <crypt.h>
#elif defined(_AIX) || defined(__hpux)
	#include <pwd.h>
	#include <crypt.h>
	#include <unistd.h>
	#include <string.h>
#elif defined(MVS)
	#include <errno.h>
	#include <pwd.h>
#elif defined(__APPLE__)
	#include <pwd.h>
	#include <unistd.h>
	#include <string.h>
#endif

#define RC_SUCCESS	100 /* Authentication succeeded */
#define RC_FAILURE	101 /* Authentication failed */
#define RC_ARG		103 /* Too few arguments */

int verifyPassword(char *cmd, char *userid, char *password);

int main(int argc, char *argv[]) {
	char *userid;
	char *password;
	int rc;

	if(argc < 3) {
		/*userid = (char*)malloc(sizeof(char) * 256);
		password = (char*)malloc(sizeof(char) * 256);
		scanf("%s", userid);
		scanf("%s", password);*/
		
		userid = (char*)malloc(sizeof(char) * 256);
		password = (char*)malloc(sizeof(char) * 256);
		
		fgets(userid, 250, stdin);
		/*printf("fgets userid: %s\n", userid);*/
		sscanf(userid,"%s", userid);
		/*printf("sscanf userid: %s\n", userid);*/
        
        fgets(password, 250, stdin);
        /*printf("fgets password: %s\n", password);*/
        sscanf(password,"%s", password);
        /*printf("sscanf password: %s\n", password);*/
        
    }
	else {
		userid = argv[1]; /* userid (argv[1]), password (argv[2]) */
		password = argv[2];
	}

	rc = verifyPassword(argv[0], userid, password);
#if _DEBUG
	if(rc == RC_SUCCESS) {
		printf("succeeded\n");
	}
	else {
		printf("failed\n");
	}
#endif
	return rc;
}

/*
 *	Linux and Solaris section
 */
#if defined(__linux__) || defined(__sparc) || defined(_SOLARISX86)
int verifyPassword(char *cmd, char *userid, char *password) {
	struct spwd *spw; /* the shadowed password structure */

	spw = getspnam(userid);
	return (spw && !strcmp(spw->sp_pwdp, crypt(password, spw->sp_pwdp))) ? RC_SUCCESS : RC_FAILURE;
}
#endif /* __linux__, __sparc */

/*
 *	AIX and HP-UX section
 */
#if defined(_AIX) || defined(__hpux) || defined(__APPLE__)
int verifyPassword(char *cmd, char *userid, char *password) {
	struct passwd *pw; /* the unshadowed but encrypted password structure */

	pw = getpwnam(userid);
	return (pw && !strcmp(pw->pw_passwd, crypt(password, pw->pw_passwd))) ? RC_SUCCESS : RC_FAILURE;
}
#endif /* _AIX, __hpux */

#if defined(MVS)
int verifyPassword(char *cmd, char *userid, char *password) {

	struct passwd *pw = getpwnam(userid);

	// no such user
	if( !pw )
		return RC_FAILURE;
	
	int rc = __passwd(pw->pw_name, password, NULL);
	if( 0 == rc )
		return RC_SUCCESS;
	
	switch( errno ) {
	case EACCES:
		fprintf( stderr, "EACCES: password not authorized\n" );
		break;

	case EINVAL:
		fprintf( stderr, "EINVAL: invalid argument(s)\n" );
		break;

	case EMVSEXPIRE:
		fprintf( stderr, "EMVSEXPIRE: password has expired\n" );
		break;

	case EMVSSAF2ERR:
		fprintf( stderr, "EMVSSAF2ERR: internal processing error\n" );
		break;

	case EMVSSAFEXTRERR:
		fprintf( stderr, "EMVSSAFEXTRERR: SAF/RACF extraction error. reason code=%d\n", __errno2() );
		break;

	case EMVSERR:
		fprintf( stderr, "EMVSERR: '%s' requires BPX.DAEMON privilege in order to verify passwords.\n", cmd );
		break;

	case ESRCH:
		fprintf( stderr, "ESRCH: user '%s' is not known to the security system or does not have an OMVS segment defined\n", userid );
		break;
		
	default:
		break;
		
	}
	return RC_FAILURE;
	
}
#endif /* MVS */
